/**
 * Required External Modules and Interfaces
 */
import express, {Request, Response} from 'express';
import {Guid} from 'guid-typescript';
import logger from '../../../middleware/logger';
import {ClimbingRoute} from './ClimbingRoute.interface';
import * as RouteService from './climbingRoutes.service';
import {verifyCaptcha} from '../common/VerifyCaptcha';

/**
 * Router Definition
 */
export const climbingRoutesRouter = express.Router();

climbingRoutesRouter.get('/', async (req: Request, res: Response) => {
    try {
        const routes: ClimbingRoute[] = await RouteService.findAll();

        res.status(200).send(routes);
    } catch (e) {
        let errorGuid = Guid.create().toString();
        logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
        res.status(500).send({
            'status': 'PROCESSING_ERROR',
            'message': 'Internal Server Error. Try again later.',
            'reference': errorGuid
        });
    }
});

climbingRoutesRouter.get('/:id', async (req: Request, res: Response) => {
    try {
        let route_id = req.params.id;

        const route: ClimbingRoute = await RouteService.findById(route_id);

        res.status(200).send(route);
    } catch (e) {
        let errorGuid = Guid.create().toString();
        logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
        res.status(500).send({
            'status': 'PROCESSING_ERROR',
            'message': 'Internal Server Error. Try again later.',
            'reference': errorGuid
        });
    }
});

climbingRoutesRouter.post('/', async (req: Request, res: Response) => {
    try {
        let gym_id = Number(req.query.gym_id);
        let name = req.query.name as string;
        let difficulty = req.query.difficulty as string;
        let captcha_token = req.query.captcha_token as string;

        if (isNaN(gym_id) || !name || !difficulty || !captcha_token) {
            res.status(400).send({'message': 'Missing parameters'});
            return;
        }

        // Verify captcha
        if (!await verifyCaptcha(captcha_token)) {
            res.status(403).send({'message': 'Invalid Captcha. Please try again.'});
            return;
        }

        let route_id = await RouteService.createRoute(gym_id, name, difficulty);

        res.status(201).send({'route_id': route_id});
    } catch (e) {
        let errorGuid = Guid.create().toString();
        logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
        res.status(500).send({
            'status': 'PROCESSING_ERROR',
            'message': 'Internal Server Error. Try again later.',
            'reference': errorGuid
        });
    }
});
